@@ -1,8 +1,9 @@
-import scrapy
-from scrapy.crawler import CrawlerProcess
 from twisted.internet.main import installReactor
 from twisted.internet.selectreactor import SelectReactor
 
+import scrapy
+from scrapy.crawler import CrawlerProcess
+
 
 class SelectReactorSubclass(SelectReactor):
     pass
@@ -13,15 +14,17 @@ installReactor(reactor)
 
 
 class NoRequestsSpider(scrapy.Spider):
-    name = 'no_request'
+    name = "no_request"
 
     def start_requests(self):
         return []
 
 
-process = CrawlerProcess(settings={
-    "TWISTED_REACTOR": "twisted.internet.selectreactor.SelectReactor",
-})
+process = CrawlerProcess(
+    settings={
+        "TWISTED_REACTOR": "twisted.internet.selectreactor.SelectReactor",
+    }
+)
 
 process.crawl(NoRequestsSpider)
 process.start()
