@@ -129,7 +129,7 @@ class FileTestCase(unittest.TestCase):
     def test_non_existent(self):
         request = Request(f"file://{self.mktemp()}")
         d = self.download_request(request, Spider("foo"))
-        return self.assertFailure(d, IOError)
+        return self.assertFailure(d, OSError)
 
 
 class ContentLengthHeaderResource(resource.Resource):
@@ -748,7 +748,7 @@ class Http11MockServerTestCase(unittest.TestCase):
         yield crawler.crawl(seed=request)
         # download_maxsize = 50 is enough for the gzipped response
         failure = crawler.spider.meta.get("failure")
-        self.assertTrue(failure is None)
+        self.assertIsNone(failure)
         reason = crawler.spider.meta["close_reason"]
         self.assertTrue(reason, "finished")
 
